@@ -1,5 +1,5 @@
 /*
- MRTG 2.17.0  -- Rateup
+ MRTG 2.17.1  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -83,7 +83,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.17.0";
+char *VERSION = "2.17.1";
 char *program, *router, *routerpath;
 int histvalid;
 
@@ -97,7 +97,6 @@ short options = 0;
 #define OPTION_NOARROW		0x0020	/* noarrow */
 #define OPTION_NO_I		0x0040	/* ignore 'I' (first) variable */
 #define OPTION_NO_O		0x0080	/* ignore 'O' (second) variable */
-#define OPTION_PNGDATE		0x0100	/* show date & time in graph */
 #define OPTION_PRINTROUTER	0x0200	/* show title in graph */
 #define OPTION_LOGGRAPH		0x0400	/* Use a logarithmic Y axis */
 #define OPTION_MEANOVER		0x0800	/* max Y = mean-above-the-mean */
@@ -276,6 +275,7 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
      int currdatetimepos;
 {
   FILE *fo;
+  char file_tmp[10240];
   int i, x, n, type;
 
   long long maxv;
@@ -286,7 +286,6 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
   double inr, outr, muli = 1, interval;
   time_t now, onow, nextnow;
   struct tm tm2, *tm = &tm2;
-  struct tm tm3, *tmtz = &tm3;
   char **graph_label;
   char ylab[30];
   /* scaling helpers */
@@ -408,15 +407,23 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
 	{
 	  maxin = (long long) max (maxin, inmax);
 	  maxout = (long long) max (maxout, outmax);
-	  maxs = (long long) max (maxs, inmax);
-	  maxs = (long long) max (maxs, outmax);
+          if (!(options & OPTION_NO_I)){
+   	     maxs = (long long) max (maxs, inmax);
+          }
+          if (!(options & OPTION_NO_O)){
+  	     maxs = (long long) max (maxs, outmax);
+          }
 	}
       else
 	{
 	  maxin = (long long) max (maxin, inr);
 	  maxout = (long long) max (maxout, outr);
-	  maxs = (long long) max (maxs, inr);
-	  maxs = (long long) max (maxs, outr);
+          if (!(options & OPTION_NO_I)){
+  	     maxs = (long long) max (maxs, inr);
+          }
+          if (!(options & OPTION_NO_O)){
+  	     maxs = (long long) max (maxs, outr);
+          }
 	}
       if ((options & OPTION_DORELPERCENT) && outr)
 	{
@@ -941,35 +948,6 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
 	}
     }
 
-  /* print the date & time */
-  if (options & OPTION_PNGDATE)
-    {
-      pngdate = (char *) calloc (1, 100);
-      tmtz = localtime (&NOW);
-      if (rtimezone == NULL)
-	{
-	  strftime (pngdate, 18, "%b %d %Y %H:%M", tmtz);
-	}
-      else
-	{
-	  strftime (pngdate, 19, "%b %d %Y %H:%M ", tmtz);
-	  if (((size_t) strlen (pngdate) + strlen (rtimezone)) < 100)
-	    {
-	      strcat (pngdate, rtimezone);
-	    }
-	  else
-	    {
-	      fprintf (stderr,
-		       "%s, Rateup ERROR: date + timezone string too long\n", bufftime);
-	      exit (1);
-	    }
-
-	}
-      gdImageString (graph, gdFontSmall,
-		     (81 + maxx * xscale) - gdFontSmall->w * strlen (pngdate),
-		     1, (unsigned char *) pngdate, i_grid);
-    }
-
   /* draw the graph border */
   gdImageRectangle (graph, xtr (0), ytr (0), xtr (maxx), ytr (maxy), i_grid);
 
@@ -1130,15 +1108,22 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
 		     currdatetimepos_x, currdatetimepos_y,
 		     (unsigned char *)currdatetimestr, i_grid);
     }
-
-  if ((fo = fopen (file, "wb")) == NULL)
+  file_tmp[0]=0;
+  strncat(file_tmp,file,1000);
+  strncat(file_tmp,".tmp",1000);
+  if ((fo = fopen (file_tmp, "wb")) == NULL)
     {
       perror (program);
-      fprintf (stderr, "%s, Rateup Error: Can't open %s for write\n", bufftime, file);
+      fprintf (stderr, "%s, Rateup Error: Can't open %s for write\n", bufftime, file_tmp);
       exit (1);
     }
   GFORM_GD (graph, fo);
   fclose (fo);
+  if (rename(file_tmp,file)){
+      perror (program);
+      fprintf (stderr, "%s, Rateup Error: Can't rename %s to %s\n", bufftime,file_tmp,file);
+      exit (1);
+  }
   gdImageDestroy (graph);
   gdImageDestroy (brush_out);
   gdImageDestroy (brush_outm);
@@ -1449,8 +1434,14 @@ update (in, out, abs_max, absupdate)
 	  else
 	    {
 	      inr = diff (in, last.in);
-	      if (inr < 0)	/* wrapped 32-bit counter? */
-		inr += (long long) 1 << 32;
+	      if (inr < 0) {
+                if (inr > - (long long) 1 << 32) { 	/* wrapped 32-bit counter? */
+  		    inr += (long long) 1 << 32;
+                }
+                else {
+  		    inr = 0;
+                }
+              }                                        
 	    }
 	  if (absupdate == 2)
 	    {
@@ -1490,8 +1481,14 @@ update (in, out, abs_max, absupdate)
 	  else
 	    {
 	      outr = diff (out, last.out);
-	      if (outr < 0)	/* wrapped 32-bit counter? */
-		outr += (long long) 1 << 32;
+	      if (outr < 0) {	/* wrapped  counter? */
+                if (outr > - (long long) 1 << 32) {
+  		    outr += (long long) 1 << 32;
+                }
+                else {
+  		    outr = 0; /* 64bit counters do not wrap usually */
+                }
+              }
 	    }
 	  if (absupdate == 2)
 	    {
@@ -1997,10 +1994,6 @@ main (argc, argv)
 	      options &= ~OPTION_NOBORDER;
 	      used = 1;
 	      break;
-	    case 'd':		/* Print date in image */
-	      options |= OPTION_PNGDATE;
-	      used = 1;
-	      break;
 	    case 'i':		/* Do not graph the I variable */
 	      options |= OPTION_NO_I;
 	      used = 1;
